/*
** Module   : Session.java
** Abstract : Interface for a network session
**
** Copyright (c) 2004-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 ECF 20071101 ADD  @35860  Created initial version. Interface for a
**                               network session.
** 002 CS  20130211              Added conversion support for appserver.
*/

package com.goldencode.p2j.net;

import java.io.*;
import com.goldencode.p2j.util.*;

/**
 * Defines the public API for a network session, including methods for low
 * level message transport, and methods for higher level session management.
 */
public interface Session
extends LowLevelSession
{
   /**
    * Add a new instance into the list of objects which will be notified
    * of session events.
    * 
    * @param   listener
    *          The listener to add.
    */
   public void addSessionListener(SessionListener listener);
   
   /**
    * Remove the listener from the list of objects to be notified about
    * session events.
    * 
    * @param   listener
    *          The listener to remove.
    */
   public void removeSessionListener(SessionListener listener);
   
   /**
    * Sets the instance of the state synchronizer the reader and writer
    * threads should use, if and only if there is no other state synchronizer
    * already in use.
    *
    * @param   sync
    *          The synchronizer to register or <code>null</code> if the
    *          currently registered synchronizer should be deregistered.
    */
   public void registerSynchronizer(StateSynchronizer sync);
   
   /**
    * Send provided message as an asynchronous request without waiting for a
    * reply.  The message type will be forced to <code>REQUEST_ASYNCH</code>.
    * 
    * @param   message
    *          Request message.
    */
   public void forward(Message message);
   
   /**
    * Send provided message as a synchronous request and wait for reply.
    * Return message payload as a result.
    * <p>
    * The message passed as a parameter must be completely filled with all
    * required information - routing key should point to valid entry point. If
    * addresses in the routing key are not set then destination is assumed at
    * directly connected node and appropriate addresses are assigned
    * automatically.
    * <p>
    * Caller can specify timeout for waiting reply message. If timeout is set
    * to 0 then queue will wait for reply as long as queue is running.
    * 
    * @param   message
    *          Request message.
    * @param   timeout
    *          Reply wait timeout.
    * 
    * @return  Reply message payload.
    * 
    * @throws  Exception
    *          Is thrown if reply received from remote side has an exception
    *          as its payload, if a timeout occurs before the reply message
    *          is received or if a problem occurs during transmission.
    */
   public Object transact(Message message, int timeout)
   throws Exception;
   
   /**
    * Terminate the network session.  This method should notify all session
    * listeners of the termination event, and will stop the underlying queue
    * if this is the only session using it.
    */
   public void terminate();
   
   /**
    * Send an echo request message using provided data as a message payload
    * and wait for reply. Return message payload as a result.
    * <p>
    * If instance of <code>RoutingKey</code> is provided then message is
    * sent to node pointed by the <code>RoutungKey</code>. Otherwise
    * directly connected node is assumed. Note that processing of the ECHO
    * message ignores entry point information in <code>RoutingKey</code> so
    * an instance of <b>RoutingKey </b> created with default constructor can
    * be used.
    * <p>
    * Caller can specify timeout for waiting reply message. If this is not
    * necessary then 0 should be passed as If timeout is set to 0 then queue
    * will wait for reply as long as queue is running.
    * 
    * @param    obj
    *           Message payload.
    * @param    timeout
    *           Reply wait timeout.
    * @param    key
    *           The destination node address information.
    * 
    * @return   Payload of the echo reply.
    * 
    * @throws   InterruptedException
    *           forwarded from transactImpl()
    * @throws   RequestTimeoutException
    *           forwarded from transactImpl()
    * @throws   ProtocolViolation
    *           in case of protocol errors.
    * @throws   Exception
    *           Is thrown if a timeout occurs before the reply message
    *           is received or if a problem occurs during transmission.
    */
   public Object echo(Serializable obj, int timeout, RoutingKey key)
   throws ProtocolViolation,
          InterruptedException,
          RequestTimeoutException,
          Exception;
   
   /**
    * Procedure that specifies the remote procedures the client can execute 
    * from the AppServer session
    * 
    * @param  attrList
    *         The list of procedure  and  name-patterns separated by comma to be
    *         exported.
    *         
    * @return <true> if REMOTE attribute is true and operation succeeded or
    *         <false> otherwise
    */
   public logical export(String attrList);
   
   /**
    * Procedure that specifies the remote procedures the client can execute 
    * from the AppServer session
    * 
    * @param  attrList
    *         The list of procedure  and  name-patterns separated by comma to be
    *         exported.
    *         
    * @return <true> if REMOTE attribute is true and operation succeeded or
    *         <false> otherwise
    */
   public logical export(character attrList);
   
   /**
    * Method implementation for getting the SERVER-OPERATING-MODE attribute of
    * the session handle.
    * <p>
    * Possible values:
    * <ul>
    *    <li>State-reset
    *    <li>State-aware
    *    <li>Stateless
    *    <li>State-free
    * </ul>
    * 
    * @return the corresponding SERVER-OPERATING-MODE attribute of the session 
    *         handle.
    */
   public character getServerOperatingMode();
   
   /**
    * Indicates if the session is an AppServer session or not
    * 
    * @return <code>true</code> if the current session is an AppServer session
    *         or false otherwise.
    */
   public logical isRemote();
   
}
